//
//  WS1Filter.h
//  VMWare
//
//  Created by Kieran Crawford on 8/20/19.
//  Copyright © 2020 VMWare. All rights reserved.
//

#ifndef WS1Filter_h
#define WS1Filter_h

#import <Foundation/Foundation.h>

#define WS1FilterTypeBlacklist WS1FilterTypeDeny

typedef NS_ENUM(NSUInteger, WS1FilterType) {
  WS1FilterTypeDeny,
  WS1FilterTypePreserveQuery,
  WS1FilterTypePreserveFragment,
  WS1FilterTypePreserveParameters,
  WS1FilterTypePreserveAll,
  WS1FilterTypeMax
};

/*!
 @class WS1Filter
 Filters can be supplied to Intelligence SDK to adjust the behavior of how URLs are
 captured and reported on for performance monitoring purposes.
 
 By default, all URLs captured by Intelligence SDK will be reported when
 performance monitoring is enabled.
 
 By default, the credentials, query, fragment, and parameters portions of all URLs will be stripped.
 
 That is, a request for the URL:
 
 https://johnny:p4ssw0rd@www.example.com:443/script.ext;param=value?query=value#ref
 
 will be reported as: https://www.example.com:443/script.ext
 
 The following filters can be added to modify this behavior.
 
 - WS1FilterTypeDeny - Deny filters: discard data pertaining to any matching URLs
 
 [WS1Filter filterWithString:@"example.com"];
 [WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypeDeny];
 
 These two methods are equivalent. Either of these filters will deny tracking of all URLs that
 match the provided string.
 
 
 - WS1FilterTypePreserveQuery - Preserves the query portion of any matching URLs
 
 [WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypePreserveQuery];
 
 Resulting URL: https://www.example.com:443/script.ext?query=value
 
 
 - WS1FilterTypePreserveParameters - Preserves the parameters portion of any matching URLs
 
 [WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypePreserveParameters];
 
 Resulting URL: https://www.example.com:443/script.ext;param=value
 
 
 - WS1FilterTypePreserveFragment - Preserves the fragment portion of any matching URLs
 
 [WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypePreserveFragment];
 
 Resulting URL: https://www.example.com:443/script.ext#ref
 
 
 - WS1FilterTypePreserveAll - Preserves the query, fragment, and parameters portion of any matching URLs
 
 [WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypePreserveAll];
 
 Resulting URL: https://www.example.com:443/script.ext;param=value?query=value#ref
 
 
 - Initializing Intelligence SDK with the above filters:
 
 [WS1Intelligence enableWithAppID:@"YOURAPPID" andConfig:config];
 [WS1Intelligence addFilter:[WS1Filter filterWithString:@"example.com" andFilterType:WS1FilterTypeDeny]];
 
 Notes:
 
 * Filters match URLs via CASE SENSITIVE substring matching
 * Filters can either be supplied when Intelligence SDK is enabled, as an array of
 WS1Filter objects, or added dynamically at any time.
 * Username and password information is always stripped provided the input URL
 is RFC 1808 compliant.
 */

NS_ASSUME_NONNULL_BEGIN

@interface WS1Filter : NSObject

@property (readonly, assign) WS1FilterType filterType;

#pragma mark - Class Methods

/*!
 *Convenience method to create a deny filter
 *@param matchToken An NSString of the url to filter
 *@return The WS1Filter for matchToken
 */
+ (WS1Filter *)filterWithString:(NSString *)matchToken;

/*!
 *Convenience method to create a filter with a certain type
 *@param matchToken An NSString of the url to filter
 *@param filterType WS1FilterType specifies the filter type
 *@return The WS1Filter for matchToken
 */
+ (WS1Filter *)filterWithString:(NSString *)matchToken
                  andFilterType:(WS1FilterType)filterType;


#pragma mark - Instance Methods

/*!
 *Initialize a filter that denies tracking of all URLs that match the specified token
 *@param matchToken NSString the of the url to filter
 *@return WS1Filter of type WS1FilterTypeDeny
 */
- (id)initWithString:(NSString *)matchToken;

/*!
 *WS1Filter designated initializer
 *@param matchToken NSString the of the url to filter
 *@param filterType WS1FilterType indicates filter type WS1FilterTypeScrubQuery, WS1FilterTypeDeny, WS1FilterTypePreserveQuery
 *@return WS1Filter with the indicated filter type
 */
- (id)initWithString:(NSString *)matchToken
       andFilterType:(WS1FilterType)filterType;

/*!
 *Checks if the url matches this filter
 *@param url NSString indicates the url to test
 *@return YES if the url matches, NO if it does not
 */
// Does specified URL match this filter?
- (BOOL)doesMatch:(NSString *)url;

@end

NS_ASSUME_NONNULL_END

#endif /* WS1Filter_h */



