//
//  WS1ServiceRegistry.m
//  WS1ServiceRegistry
//
//  Created by Edward Yu on 10/17/18.
//  Copyright © 2018 air-watch. All rights reserved.
//

#import "WS1ServiceRegistry.h"
#import "WS1ServiceContainer.h"
#import "WS1ServiceRegistryProtocol.h"


NS_ASSUME_NONNULL_BEGIN
@interface WS1ServiceRegistry () <WS1ServiceRegistryProtocol>
@property (nonatomic) NSMutableDictionary<NSString *, NSMutableArray<id<WS1ServiceRegistryObserver>> *> *serviceObservers;
@property (nonatomic) NSMutableDictionary<NSString *, WS1ServiceContainer*> *serviceContainers;
@end

@implementation WS1ServiceRegistry

#pragma mark - Public methods to enforce usage of shared singleton

+ (void)registerService:(NSObject *)service
         forServiceType:(Protocol *)serviceType
         withIdentifier:(NSString *)serviceIdentifier
         withCompletion:(void (^)(NSError * _Nullable))completionHandler {

    [[WS1ServiceRegistry sharedRegistry] registerService:(id)service
                                          forServiceType:serviceType
                                          withIdentifier:serviceIdentifier
                                          withCompletion:completionHandler];
}

+ (void)unregisterServiceOfType:(nonnull Protocol *)serviceType
                 withIdentifier:(nonnull NSString *)serviceIdentifier
                   warnDataLoss:(BOOL)warnDataLoss
                 withCompletion:(nonnull void (^)(NSError * _Nullable))completionHandler {

    [[WS1ServiceRegistry sharedRegistry] unregisterServiceOfType:serviceType
                                                  withIdentifier:serviceIdentifier
                                                    warnDataLoss:warnDataLoss
                                                  withCompletion:completionHandler];
}


+ (NSObject *_Nullable)fetchServiceOfType:(Protocol *)serviceType
                                    error:(NSError **)errorPtr {

    return [[WS1ServiceRegistry sharedRegistry] fetchServiceOfType:serviceType
                                                             error:errorPtr];

}

+ (BOOL)addObserver:(id <WS1ServiceRegistryObserver>)observer
     forServiceType:(Protocol *)serviceType
              error:(NSError **)errorPtr {

    return [[WS1ServiceRegistry sharedRegistry] addObserver:observer
                                             forServiceType:serviceType
                                                      error:errorPtr];
}

+ (BOOL)removeObserver:(id <WS1ServiceRegistryObserver>)observer
            forServiceType:(Protocol *)serviceType
                     error:(NSError **)errorPtr {

    return [[WS1ServiceRegistry sharedRegistry] removeObserver:observer
                                                forServiceType:serviceType
                                                         error:errorPtr];

}



#pragma mark - Shared singleton object
+ (instancetype)sharedRegistry {
    static WS1ServiceRegistry *shared = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        shared = [[self alloc] init];
    });
    return shared;
}

#pragma mark - Instance methods
- (instancetype)init {
    if (self = [super init]) {
        self.serviceContainers = [NSMutableDictionary dictionary];
        self.serviceObservers = [NSMutableDictionary dictionary];
    }
    return self;
}

// For registering the service
// Registry will hold strong reference to service until unregistered
- (void)registerService:(id)service
         forServiceType:(Protocol *)serviceType
         withIdentifier:(NSString *)serviceIdentifier
         withCompletion:(void (^)(NSError * _Nullable))completionHandler {

    NSString *serviceTypeString = NSStringFromProtocol(serviceType);
    if (![(NSObject *)service conformsToProtocol: serviceType]) {

        // Service doesn't conform to provided service type
        NSError *error = [self generateErrorWithCode:WS1ServiceRegistryErrorServiceNotConforming
                                    debugDescription:[NSString stringWithFormat:@"Service doesn't conform to provided serviceType %@", serviceTypeString]];
        completionHandler(error);
        return;
    }

    if (self.serviceContainers[serviceTypeString] != nil) {
        // Service type already registered, cannot register a new one
        NSError *error = [self generateErrorWithCode:WS1ServiceRegistryErrorServiceTypeAlreadyRegistered
                   debugDescription:[NSString stringWithFormat:@"Service with type %@ already registered", serviceTypeString]];
        completionHandler(error);
        return;
    }

    // Register the service
    WS1ServiceContainer *container = [WS1ServiceContainer containerWithService:service andIdentifier:serviceIdentifier];
    self.serviceContainers[serviceTypeString] = container;

    // Inform the service of its successful registration.
    // NOTE: Must call completion handler before notifying any observers.
    // The objects or classes registering their service may need to do work after successful registration is confirmed, but before
    // other objects attempt to use their service. Once execution of the completionHandler is finished, the service is expected
    // to be ready and available.
    NSLog(@"[WS1ServiceRegistry]: call completionHandler for registration success");
    completionHandler(nil);
    
    // Broadcast successful registration to observers
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^{
        for (id<WS1ServiceRegistryObserver>observer in self.serviceObservers[serviceTypeString]) {
            [observer serviceRegistered:serviceType];
        }
        NSLog(@"[WS1ServiceRegistry]: Successfully notified observers of the registration of %@ typed service", serviceTypeString);
    });
    NSLog(@"[WS1ServiceRegistry]: Successfully registered %@ typed service", serviceTypeString);
}

// For unregistering the service
- (void)unregisterServiceOfType:(Protocol *)serviceType
                 withIdentifier:(NSString *)serviceIdentifier
                   warnDataLoss:(BOOL)warnDataLoss
                 withCompletion:(void (^)(NSError * _Nullable))completionHandler {
    
    NSString *serviceTypeString = NSStringFromProtocol(serviceType);
    WS1ServiceContainer *storedContainer = self.serviceContainers[serviceTypeString];
    if (storedContainer == nil) {
        // Service not found to unregister
        NSError *error = [self generateErrorWithCode:WS1ServiceRegistryErrorServiceNotFound
                   debugDescription:[NSString stringWithFormat:@"Service with type %@ not found", serviceTypeString]];
        completionHandler(error);
        return;
    }
    if (![serviceIdentifier isEqualToString:storedContainer.identifier]) {
        // Incorrect identifier, do not allow unregister
        // Surface as a service not found error
        NSError *error = [self generateErrorWithCode:WS1ServiceRegistryErrorServiceNotFound
                   debugDescription:[NSString stringWithFormat:@"Service with type %@ not found", serviceTypeString]];
        completionHandler(error);
        return;
    }
    // Unregister the service
    self.serviceContainers[serviceTypeString] = nil;

    // Broadcast the successful unregister to observers
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^{
        for (id<WS1ServiceRegistryObserver>observer in self.serviceObservers[serviceTypeString]) {
            [observer serviceUnregistered:serviceType withDataLoss:warnDataLoss];
        }
        NSLog(@"[WS1ServiceRegistry]: Successfully notified observers of the un-registration of %@ typed service", serviceTypeString);
        completionHandler(nil);
    });
    NSLog(@"[WS1ServiceRegistry]: Successfully unregistered %@ typed service", serviceTypeString);
}

// For fetching a previously registered service
// Caller should not hold onto fetched reference of service
- (id _Nullable)fetchServiceOfType:(Protocol *)serviceType
                             error:(NSError **)errorPtr {
    NSString *serviceTypeString = NSStringFromProtocol(serviceType);
    id service = self.serviceContainers[serviceTypeString].service;

    if (!service) {
        // Service is not found
        [self generateErrorWithCode:WS1ServiceRegistryErrorServiceNotFound
                   debugDescription:[NSString stringWithFormat:@"Service with type %@ not found", serviceTypeString]];
        return nil;
    }

    // Future improvement: return a proxy object so reference to service can be cut off when it is unregistered
    NSLog(@"[WS1ServiceRegistry]: Successfully fetched %@ typed service", serviceTypeString);
    return service;
}

// For adding an observer to get informed about the registrating/unregistering of the service type
- (BOOL)addObserver:(id <WS1ServiceRegistryObserver>)observer
     forServiceType:(Protocol *)serviceType
              error:(NSError **)errorPtr {

    if (![self isObserverValid:observer]) {
        // Observer not conforming to WS1ServiceRegistryObserver type
         *errorPtr = [self generateErrorWithCode:WS1ServiceRegistryErrorObserverInvalid
                   debugDescription:@"Observer does not conform to the WS1ServiceRegistryObserver type"];
        return NO;
    }


    NSString *serviceTypeString = NSStringFromProtocol(serviceType);
    NSMutableArray<id<WS1ServiceRegistryObserver>> *observers = self.serviceObservers[serviceTypeString];
    if (observers == nil) {
        // Array hasn't been created yet
        observers = [NSMutableArray array];
        self.serviceObservers[serviceTypeString] = observers;
    }

    // Register the observer
    [observers addObject:(id <WS1ServiceRegistryObserver>)observer];
    NSLog(@"[WS1ServiceRegistry]: Successfully registered observer for %@", serviceTypeString);

    return YES;
}

// For removing observers
// Should be paired with number of addObserver: calls
// If the same observer is added twice, removeObserver: should be called twice
- (BOOL)removeObserver:(id <WS1ServiceRegistryObserver>)observer
        forServiceType:(Protocol *)serviceType
                 error:(NSError **)errorPtr {

    if (![self isObserverValid:observer]) {
        // Observer not conforming to WS1ServiceRegistryObserver type
         *errorPtr = [self generateErrorWithCode:WS1ServiceRegistryErrorObserverInvalid
                   debugDescription:@"Observer does not conform to the WS1ServiceRegistryObserver type"];
        return NO;
    }

    NSString *serviceTypeString = NSStringFromProtocol(serviceType);
    NSMutableArray<id<WS1ServiceRegistryObserver>> *observers = self.serviceObservers[serviceTypeString];
    if (observers == nil) {
        // No existing observers for service type
         *errorPtr = [self generateErrorWithCode:WS1ServiceRegistryErrorObserverNotFound
                   debugDescription:[NSString stringWithFormat:@"Observer for service type %@ not found", serviceTypeString]];
        return NO;
    }

    // Search for specific observer
    NSUInteger index = [observers indexOfObjectIdenticalTo:(id <WS1ServiceRegistryObserver>)observer];
    if (index == NSNotFound) {
        // Observer not found
         *errorPtr = [self generateErrorWithCode:WS1ServiceRegistryErrorObserverNotFound
                   debugDescription:[NSString stringWithFormat:@"Observer for service type %@ not found", serviceTypeString]];
        return NO;
    }

    // Remove the observer
    [observers removeObjectAtIndex:index];
    NSLog(@"[WS1ServiceRegistry]: Successfully unregistered observer for %@", serviceTypeString);
    return YES;
}

// Helper method for checking if observer is valid
- (BOOL)isObserverValid:(NSObject *)observer {
    if ([observer respondsToSelector:@selector(serviceRegistered:)] &&
        [observer respondsToSelector:@selector(serviceUnregistered:withDataLoss:)]) {
        return YES;
    } else {
        return NO;
    }
}


// Helper method to generate and log errors
- (NSError*)generateErrorWithCode:(NSInteger)code debugDescription:(NSString *)description {
    NSLog(@"[WS1ServiceRegistry]: %@", description);
    NSMutableDictionary<NSErrorUserInfoKey,id> *userInfo = [NSMutableDictionary dictionary];
    userInfo[NSDebugDescriptionErrorKey] = description;
    return [NSError errorWithDomain:WS1ServiceRegistryErrorDomain code:code userInfo:userInfo];
}

@end
NS_ASSUME_NONNULL_END
