//
//  WS1ServiceRegistryObserver.h
//  WS1ServiceRegistryProtocol
//
//  Created by Edward Yu on 2018-12-13.
//  Copyright © 2018 VMware. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN
/**
 `WS1ServiceRegistryObserver` protocol describes an observer which will get informed when `WS1ServiceRegistry` registers/unregisters
 a particular service type (Protocol *).

 ## Usage
 Those interested whether @protocol(`XService`) is registered/unregistered with `WS1ServiceRegistry` should implement an observer object conforming to
 `WS1ServiceRegistryObserver` which can handle the method calls with the parameter @protocol(`XService`). Then add the observer object to the registry,
 using `+[WS1ServiceRegistry addObserver:forServiceType:error:]`. `WS1ServiceRegistry` will then call `-[serviceRegistered:]` and
 `-[serviceUnregistered:withDataLoss:]` as appropriate.
 */
@protocol WS1ServiceRegistryObserver <NSObject>

/**
 * Inform the observer that a service typed serviceType was registered with `WS1ServiceRegistry`
 * @param @"serviceType" the service type that was registered
 * @see +[WS1ServiceRegistry registerService:forServiceType:withIdentifier:error:]
 */
- (void)serviceRegistered:(Protocol *)serviceType;

/**
 * Inform the observer that a service typed serviceType was unregistered with `WS1ServiceRegistry`, passing along the data loss flag in the unregister call
 * @param @"serviceType" the service type that was unregistered
 * @param @"dataLoss" whether the service was unregistered with data loss
 * @see +[WS1ServiceRegistry unregisterServiceOfType:withIdentifier:warnDataLoss:error:]
 */
- (void)serviceUnregistered:(Protocol *)serviceType
               withDataLoss:(BOOL)dataLoss;
@end

NS_ASSUME_NONNULL_END
