//
//  WS1ServiceRegistryProtocol.h
//  WS1ServiceRegistryProtocol
//
//  Created by Edward Yu on 2018-11-08.
//  Copyright © 2018 VMware. All rights reserved.
//

#import <Foundation/Foundation.h>

//! Project version number for WS1ServiceRegistryProtocol.
FOUNDATION_EXPORT double WS1ServiceRegistryProtocolVersionNumber;

//! Project version string for WS1ServiceRegistryProtocol.
FOUNDATION_EXPORT const unsigned char WS1ServiceRegistryProtocolVersionString[];

#import <WS1ServiceRegistryProtocol/WS1ServiceRegistryObserver.h>

NS_ASSUME_NONNULL_BEGIN

/**
 `WS1ServiceRegistryErrorDomain` is the error domain for errors of WS1ServiceRegistry.
 */
extern NSString *const WS1ServiceRegistryErrorDomain;

/**
 `WS1ServiceRegistryError` is the error enum describing the possible errors of WS1ServiceRegistry.
 */
typedef NS_ERROR_ENUM(WS1ServiceRegistryErrorDomain, WS1ServiceRegistryError) {
    WS1ServiceRegistryErrorServiceNotConforming = 0,
    WS1ServiceRegistryErrorServiceTypeAlreadyRegistered = 1,
    WS1ServiceRegistryErrorServiceNotFound = 2,
    WS1ServiceRegistryErrorObserverInvalid = 3,
    WS1ServiceRegistryErrorObserverNotFound = 4
};

/**
 `WS1ServiceRegistryProtocol` protocol describes the public methods of WS1ServiceRegistry, which adopts this protocol.
 */
@protocol WS1ServiceRegistryProtocol <NSObject>

/**
 For registering the service.
 Only one service of the same `serviceType` can be registered at a time.
 Registry will hold strong reference to service until unregistered.

 Registry will call `-[serviceRegistered:]` on `WS1ServiceRegistryObserver`s for `serviceType` which are added to the registry.

 @param service The actual service object to be registered.
 @param serviceType The protocol that this service should be registered as.
 @param serviceIdentifier A string to identify the service, used to check whether unregistration is allowed when unregistering the service.
 @param completionHandler Completion block to be executed once the registration has completed; will pass back error if any error occured.
 @return Whether the service registration was successful
 */
+ (void)registerService:(NSObject *)service
         forServiceType:(Protocol *)serviceType
         withIdentifier:(NSString *)serviceIdentifier
         withCompletion:(void (^)(NSError * _Nullable error))completionHandler;

/**
 For unregistering the service.
 Will not succeed if `serviceIdenifier` is not equal to the identifier provided when registering the corresponding service.

 Registry will call `-[serviceUnregistered:withDataLoss:]` on `WS1ServiceRegistryObserver`s for `serviceType` which are added to the registry.
 If `warnDataLoss` of this method is set to true, the service unregistered message to the observer will have withDataLoss set to true.

 @param service The actual service object to be unregistered.
 @param serviceIdentifier A string to identify the service, used to check whether unregistration is allowed.
 @param warnDataLoss Whether Registry should pass on a data-loss flag when calling `-[serviceUnregistered:withDataLoss:]` on the observers.
 @param completionHandler Completion block to be executed once the un-registration has completed; will pass back error if any error occured.
 @return Whether the service unregistration was successful
 */
+ (void)unregisterServiceOfType:(Protocol *)serviceType
                 withIdentifier:(NSString *)serviceIdentifier
                   warnDataLoss:(BOOL)warnDataLoss
                 withCompletion:(void (^)(NSError * _Nullable error))completionHandler;;

/**
 For fetching a previously registered service. Caller should NEVER hold onto fetched reference of service.

 @param serviceType The desired service type protocol for which the service should be fetched.
 @param errorPtr Pointer to an error object. Pointer is set to an actual error object if there is an error.
 @return If successful, a fetched reference of the actual service. Caller should NEVER hold onto fetched reference of service.
 */
+ (NSObject *_Nullable)fetchServiceOfType:(Protocol *)serviceType
                                    error:(NSError **)errorPtr;

/**
 For adding an observer to get informed about the registrating/unregistering of the service type.
 Registry will hold strong reference to the observer until it is removed.

 Should be paired with the number of of `+[removeObserver:forServiceType:error:]` calls.
 If the same observer is added twice, it should be removed twice

 @param observer The observer object that should be added to the registry.
 @param serviceType The desired service type protocol for which the observer should be informed about.
 @param errorPtr Pointer to an error object. Pointer is set to an actual error object if there is an error.
 @return Whether adding the observer was successful
 */
+ (BOOL)addObserver:(id <WS1ServiceRegistryObserver>)observer
     forServiceType:(Protocol *)serviceType
              error:(NSError **)errorPtr;

/**
 For removing a previously added observer from the Registry.

 Should be paired with the number of of `+[addObserver:forServiceType:error:]` calls.
 If the same observer is added twice, it should be removed twice

 @param observer The observer object that should be removed from the registry.
 @param serviceType The desired service type protocol which the observer was previously added for.
 @param errorPtr Pointer to an error object. Pointer is set to an actual error object if there is an error.
 @return Whether removing the observer was successful
 */
+ (BOOL)removeObserver:(id <WS1ServiceRegistryObserver>)observer
        forServiceType:(Protocol *)serviceType
                 error:(NSError **)errorPtr;

@end
NS_ASSUME_NONNULL_END
